"use strict";
/* eslint-disable @typescript-eslint/no-explicit-any */
Object.defineProperty(exports, "__esModule", { value: true });
exports.persistStore = exports.persistReducer = exports.sessionStorage = exports.localStorage = exports.PERSIST = exports.PAUSE = exports.PURGE = exports.REHYDRATE = void 0;
exports.REHYDRATE = 'persist/rehydrate';
exports.PURGE = 'persist/purge';
exports.PAUSE = 'persist/pause';
exports.PERSIST = 'persist/persist';
exports.localStorage = createAsyncStorage(globalThis.localStorage);
exports.sessionStorage = createAsyncStorage(globalThis.sessionStorage);
const PERSISTORS = [];
const persistReducer = (config, reducer) => {
    const { key, storage, stringify, throttle, merge } = config;
    const stringify_ = stringify || JSON.stringify;
    const merge_ = merge || ((stored) => stored);
    PERSISTORS.push(config);
    let timeout = null;
    let toSave = undefined;
    let isPaused = true;
    function persist(state) {
        toSave = state;
        if (!timeout)
            timeout = setTimeout(storeState, throttle || 0);
    }
    async function storeState() {
        const state = toSave;
        await storage.setItem(key, stringify_(state));
        timeout = null;
        if (state !== toSave)
            persist(toSave);
        return state;
    }
    return (state, action) => {
        if (action.persist !== undefined && action.persist === key) {
            switch (action.type) {
                case exports.REHYDRATE:
                    if (action.payload !== undefined)
                        state = merge_(action.payload, state);
                    isPaused = false;
                    break;
                case exports.PURGE:
                    persist(undefined);
                    return state;
                case exports.PAUSE:
                    isPaused = true;
                    break;
                case exports.PERSIST:
                    isPaused = false;
                    break;
                default:
                    break;
            }
        }
        const newState = reducer(state, action);
        if (state != newState && !isPaused)
            persist(newState);
        return newState;
    };
};
exports.persistReducer = persistReducer;
const persistStore = (store) => {
    // Rehydrate
    for (const cfg of PERSISTORS) {
        const { storage, parse, key } = cfg;
        const parse_ = parse || JSON.parse;
        storage.getItem(key).then((stored) => {
            store.dispatch({
                type: exports.REHYDRATE,
                payload: stored ? parse_(stored) : undefined,
                persist: key,
            });
        });
    }
    return {
        purge: (key) => dispatchByKey(store.dispatch, exports.PURGE, key),
        pause: (key) => dispatchByKey(store.dispatch, exports.PAUSE, key),
        persist: (key) => dispatchByKey(store.dispatch, exports.PERSIST, key),
    };
};
exports.persistStore = persistStore;
const dispatchByKey = (dispatch, type, key) => {
    for (const cfg of PERSISTORS) {
        if (key === undefined || cfg.key === key)
            dispatch({ type, persist: key });
    }
};
function createAsyncStorage(storage) {
    return {
        setItem: (key, value) => new Promise((resolve) => {
            resolve(storage.setItem(key, value));
        }),
        getItem: (key) => new Promise((resolve) => resolve(storage.getItem(key))),
    };
}
