/* eslint-disable @typescript-eslint/no-explicit-any */
var __awaiter =
  (this && this.__awaiter) ||
  function (thisArg, _arguments, P, generator) {
    function adopt(value) {
      return value instanceof P
        ? value
        : new P(function (resolve) {
            resolve(value)
          })
    }
    return new (P || (P = Promise))(function (resolve, reject) {
      function fulfilled(value) {
        try {
          step(generator.next(value))
        } catch (e) {
          reject(e)
        }
      }
      function rejected(value) {
        try {
          step(generator['throw'](value))
        } catch (e) {
          reject(e)
        }
      }
      function step(result) {
        result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected)
      }
      step((generator = generator.apply(thisArg, _arguments || [])).next())
    })
  }
export const REHYDRATE = 'persist/rehydrate'
export const PURGE = 'persist/purge'
export const PAUSE = 'persist/pause'
export const PERSIST = 'persist/persist'
export const localStorage = createAsyncStorage(globalThis.localStorage)
export const sessionStorage = createAsyncStorage(globalThis.sessionStorage)
const PERSISTORS = []
export const persistReducer = (config, reducer) => {
  const { key, storage, stringify, throttle, merge } = config
  const stringify_ = stringify || JSON.stringify
  const merge_ = merge || ((stored) => stored)
  PERSISTORS.push(config)
  let timeout = null
  let toSave = undefined
  let isPaused = true
  function persist(state) {
    toSave = state
    if (!timeout) timeout = setTimeout(storeState, throttle || 0)
  }
  function storeState() {
    return __awaiter(this, void 0, void 0, function* () {
      const state = toSave
      yield storage.setItem(key, stringify_(state))
      timeout = null
      if (state !== toSave) persist(toSave)
      return state
    })
  }
  return (state, action) => {
    if (action.persist !== undefined && action.persist === key) {
      switch (action.type) {
        case REHYDRATE:
          if (action.payload !== undefined) state = merge_(action.payload, state)
          isPaused = false
          break
        case PURGE:
          persist(undefined)
          return state
        case PAUSE:
          isPaused = true
          break
        case PERSIST:
          isPaused = false
          break
        default:
          break
      }
    }
    const newState = reducer(state, action)
    if (state != newState && !isPaused) persist(newState)
    return newState
  }
}
export const persistStore = (store) => {
  // Rehydrate
  for (const cfg of PERSISTORS) {
    const { storage, parse, key } = cfg
    const parse_ = parse || JSON.parse
    storage.getItem(key).then((stored) => {
      store.dispatch({
        type: REHYDRATE,
        payload: stored ? parse_(stored) : undefined,
        persist: key,
      })
    })
  }
  return {
    purge: (key) => dispatchByKey(store.dispatch, PURGE, key),
    pause: (key) => dispatchByKey(store.dispatch, PAUSE, key),
    persist: (key) => dispatchByKey(store.dispatch, PERSIST, key),
  }
}
const dispatchByKey = (dispatch, type, key) => {
  for (const cfg of PERSISTORS) {
    if (key === undefined || cfg.key === key) dispatch({ type, persist: key })
  }
}
function createAsyncStorage(storage) {
  return {
    setItem: (key, value) =>
      new Promise((resolve) => {
        resolve(storage.setItem(key, value))
      }),
    getItem: (key) => new Promise((resolve) => resolve(storage.getItem(key))),
  }
}
