/* eslint-disable @typescript-eslint/ban-ts-comment */
// @ts-nocheck
import produce, { isDraft, isDraftable } from 'immer'
/**
 * A helper to create reducers (uses immer)
 */
export const createReducer = function (DEFAULT, ...mixins) {
  const reducer = createBaseReducer(DEFAULT, ...mixins)
  return function (state = DEFAULT, action) {
    if (isDraft(state) || !isDraftable(state)) return reducer(state, action)
    return produce(state, (draft) => reducer(draft, action))
  }
}
/**
 * A helper to create reducers (without immer)
 */
export const createBaseReducer = function (DEFAULT, ...mixins) {
  const reducers = Object.assign({}, ...mixins)
  return function (state = DEFAULT, action) {
    const producer = reducers[action.type]
    if (producer) state = producer(state, action)
    return state
  }
}
